@@ -31,8 +31,8 @@
 
 # TODO Need to fix ASAP conditions in e4 not comma delimited
 
-__version__ = '1.3.7'
-__date__ = 'July, 2022'
+__version__ = '1.3.10'
+__date__ = 'January, 2023'
 __program__ = 'E5'
 
 # region Imports
@@ -79,7 +79,7 @@ import logging
 from e5py.lib.blockdata import blockdata
 from e5py.lib.dbs import dbs
 from e5py.lib.e5_widgets import e5_MainScreen, e5_scrollview_menu, e5_scrollview_label, e5_button, e5_label, e5_side_by_side_buttons, e5_LoadDialog, e5_MessageBox, e5_textinput, e5_RecordEditScreen
-from e5py.lib.e5_widgets import e5_DatagridScreen, e5_InfoScreen, e5_LogScreen, e5_CFGScreen, e5_INIScreen, e5_SettingsScreen
+from e5py.lib.e5_widgets import e5_DatagridScreen, e5_InfoScreen, e5_LogScreen, e5_CFGScreen, e5_INIScreen, e5_SettingsScreen, DataUploadScreen
 from e5py.lib.colorscheme import ColorScheme
 from e5py.lib.misc import platform_name, restore_window_size_position, locate_file, filename_only
 
@@ -87,11 +87,11 @@ from e5py.lib.misc import platform_name, restore_window_size_position, locate_fi
 from tinydb import where
 from tinydb import __version__ as __tinydb_version__
 
-
 # endregion
 
 # region Data Classes
 
+
 class db(dbs):
     MAX_FIELDS = 300
     datagrid_doc_id = None
@@ -135,7 +135,7 @@ class ini(blockdata):
                 elif test > 200:
                     test = 200
                 self.update_value(__program__, 'BACKUPINTERVAL', test)
-            except:
+            except ValueError:
                 self.update_value(__program__, 'BACKUPINTERVAL', 0)
         else:
             self.update_value(__program__, 'BACKUPINTERVAL', 0)
@@ -143,6 +143,8 @@ class ini(blockdata):
     def update(self, e5_colors, e5_cfg):
         self.update_value(__program__, 'CFG', e5_cfg.filename)
         self.update_value(__program__, 'ColorScheme', e5_colors.color_scheme)
+        self.update_value(__program__, 'ButtonFontSize', e5_colors.button_font_size)
+        self.update_value(__program__, 'TextFontSize', e5_colors.text_font_size)
         self.update_value(__program__, 'DarkMode', 'TRUE' if e5_colors.darkmode else 'FALSE')
         self.update_value(__program__, 'IncrementalBackups', self.incremental_backups)
         self.update_value(__program__, 'BackupInterval', self.backup_interval)
@@ -269,7 +271,7 @@ class cfg(blockdata):
         if data_to_insert and data_table and len(data_to_insert) == 1:
             for field, value in data_to_insert.items():
                 f = self.get(field)
-                if f.required and value.strip() == "":
+                if f.required and str(value).strip() == "":
                     error_message = f'\nThe field {field} is set to unique or required.  Enter a value to save this record.'
                     return(error_message)
                 if f.inputtype == 'NUMERIC':
@@ -631,6 +633,9 @@ class cfg(blockdata):
                     condition = condition[:-3]
                     condition_parsed.append(condition.strip())
                     condition_parsed.append('OR')
+                if condition.upper().endswith(' AND'):
+                    condition = condition[:-4]
+                    condition_parsed.append(condition.strip())
                 else:
                     condition_parsed.append(condition.strip())
                 condition_parsed = self.clean_menu(condition_parsed)
@@ -776,10 +781,20 @@ class cfg(blockdata):
                 csv_row = ''
                 for fieldname in cfg_fields:
                     if fieldname in row.keys():
-                        if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
-                            csv_row += ',%s' % row[fieldname] if csv_row else "%s" % row[fieldname]
+                        if row[fieldname] is not None:
+                            if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
+                                csv_row += ',%s' % row[fieldname] if csv_row else "%s" % row[fieldname]
+                            else:
+                                csv_row += ',"%s"' % row[fieldname] if csv_row else '"%s"' % row[fieldname]
                         else:
-                            csv_row += ',"%s"' % row[fieldname] if csv_row else '"%s"' % row[fieldname]
+                            if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
+                                if csv_row:
+                                    csv_row = csv_row + ','     # Not sure this works if there is an entirely empty row of numeric values
+                            else:
+                                if csv_row:
+                                    csv_row = csv_row + ',""'
+                                else:
+                                    csv_row = '""'
                     else:
                         if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
                             if csv_row:
@@ -792,7 +807,7 @@ class cfg(blockdata):
                 f.write(csv_row + '\n')
             f.close()
             return(None)
-        except:
+        except OSError:
             return('\nCould not write data to %s.' % (filename))
 
     def write_geojson(self, filename, table):
@@ -812,11 +827,12 @@ class cfg(blockdata):
                 comma = ' '
                 for fieldname in cfg_fields:
                     if fieldname in row.keys():
-                        geojson_row += comma + '"%s": ' % fieldname
-                        if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
-                            geojson_row += '%s' % row[fieldname]
-                        else:
-                            geojson_row += '"%s"' % row[fieldname]
+                        if row[fieldname] != '':
+                            geojson_row += comma + '"%s": ' % fieldname
+                            if self.get(fieldname).inputtype in ['NUMERIC', 'INSTRUMENT']:
+                                geojson_row += '%s' % row[fieldname]
+                            else:
+                                geojson_row += '"%s"' % row[fieldname]
                     comma = ', '
                 geojson_row += ' },\n'
                 geojson_row += '"geometry": { "type": "Point", "coordinates": [ '
@@ -944,6 +960,10 @@ class MainScreen(e5_MainScreen):
                                     ini = self.ini))
         sm.add_widget(AboutScreen(name = 'AboutScreen',
                                         colors = self.colors))
+        sm.add_widget(DataUploadScreen(name = 'UploadScreen',
+                                        data = self.data,
+                                        cfg = self.cfg,
+                                        colors = self.colors))
         sm.add_widget(EditLastRecordScreen(name = 'EditLastRecordScreen',
                                             data = self.data,
                                             doc_id = None,
@@ -1159,48 +1179,56 @@ class MainScreen(e5_MainScreen):
     #    self._keyboard = None
 
     def _on_keyboard_down(self, *args):
-        ascii_code = args[1]
-        text_str = args[3]
-        # print('INFO: The key %s has been pressed %s' % (ascii_code, text_str))
-        if not self.popup_open:
-            if ascii_code in [8, 127]:
-                return False
-            if ascii_code == 9:
-                if self.widget_with_focus.id == 'menu_scroll':
-                    self.widget_with_focus = self.field_data
-                    self.widget_with_focus.focus = True
-                elif self.widget_with_focus.id == 'field_data' and self.cfg.current_field.inputtype in ['MENU', 'BOOLEAN']:
-                    self.widget_with_focus.focus = False
-                    self.widget_with_focus = self.get_widget_by_id('menu_scroll')
-                return False
-            if ascii_code == 27:
-                if self.cfg.filename:
-                    self.go_back(None)
-            if ascii_code == 13:
-                if self.cfg.filename:
-                    self.go_next(None)
-                elif self.cfg_files:
-                    self.cfg_selected(self.scroll_menu.scroll_menu_get_selected())
-            if ascii_code == 51:
-                return True
-            if ascii_code in [273, 274, 275, 276, 278, 279] and self.scroll_menu:
-                self.scroll_menu.move_scroll_menu_item(ascii_code)
-                return False
-            if ascii_code in [276, 275, 278, 279] and not self.scroll_menu:
-                return False
-            # if text_str:
-            #    if text_str.upper() in ascii_uppercase:
-            #        self.add_scroll_content(self.get_widget_by_id('scroll_content'),
-            #                                self.get_widget_by_id('field_data').text + text_str)
-            #        return True
+        if self.parent:
+            if self.parent.current == 'MainScreen':
+                ascii_code = args[1]
+                text_str = args[3]
+                # print('INFO: The key %s has been pressed %s' % (ascii_code, text_str))
+                if not self.popup_open:
+                    if ascii_code in [8, 127]:
+                        return False
+                    if ascii_code == 9:
+                        if self.widget_with_focus.id == 'menu_scroll':
+                            self.widget_with_focus = self.field_data
+                            self.widget_with_focus.focus = True
+                        elif self.widget_with_focus.id == 'field_data' and self.cfg.current_field.inputtype in ['MENU', 'BOOLEAN']:
+                            self.widget_with_focus.focus = False
+                            self.widget_with_focus = self.get_widget_by_id('menu_scroll')
+                        return False
+                    if ascii_code == 27:
+                        if self.cfg.filename:
+                            self.go_back(None)
+                    if ascii_code == 13:
+                        if self.cfg.filename:
+                            self.go_next(None)
+                        elif self.cfg_files:
+                            self.cfg_selected(self.scroll_menu.scroll_menu_get_selected())
+                    if ascii_code == 51:
+                        return True
+                    if ascii_code in [273, 274, 275, 276, 278, 279] and self.scroll_menu:
+                        self.scroll_menu.move_scroll_menu_item(ascii_code)
+                        return False
+                    if ascii_code in [276, 275, 278, 279] and not self.scroll_menu:
+                        return False
+                    #if 'ctrl' in args[4] and args[3] == 'c':
+                    #    Clipboard.copy(self.field_data.text)
+                    #if 'ctrl' in args[4] and args[3] == 'v':
+                    #    self.field_data.text = Clipboard.paste()
+                    # if text_str:
+                    #    if text_str.upper() in ascii_uppercase:
+                    #        self.add_scroll_content(self.get_widget_by_id('scroll_content'),
+                    #                                self.get_widget_by_id('field_data').text + text_str)
+                    #        return True
+                else:
+                    if ascii_code == 13:
+                        self.close_popup(None)
+                        self.widget_with_focus.focus = True
+                        return False
+                    elif ascii_code in [275, 276]:
+                        return(False)
+                return False  # return True to accept the key. Otherwise, it will be used by the system.
         else:
-            if ascii_code == 13:
-                self.close_popup(None)
-                self.widget_with_focus.focus = True
-                return False
-            elif ascii_code in [275, 276]:
-                return(False)
-        return True  # return True to accept the key. Otherwise, it will be used by the system.
+            return False
 
     def copy_from_menu_to_textbox(self):
         if self.cfg.current_field.inputtype in ['MENU', 'BOOLEAN']:
